/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: GUID Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Guid.h,v 1.4 2008/01/07 16:58:29 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_GUID_H_
#define GLITE_DATA_AGENTS_GUID_H_

#include <string>
#include <uuid/uuid.h>

namespace glite {
namespace data  {
namespace agents{

/**
 * Guid Class
 */
struct Guid{
    /**
     * Default Constructor
     */
    Guid();
    /**
     * Constructor
     */
    Guid(const std::string& str);
    /**
     * Copy Constructor
     */
    Guid(const Guid& other);
    /**
     * Assignment Operator
     */
    void operator=(const Guid& other) /*throw()*/;

    /**
     * Comparison Operator
     */
    bool operator==(const Guid& other) const /*throw ()*/;

    /**
     * Comparison Operator
     */
    bool operator<(const Guid& other) const /*throw ()*/;

    /**
     * Comparison Operator
     */
    bool operator>(const Guid& other) const /*throw ()*/;

    /**
     * Check if the Guid is Null
     */
    bool isNull() const /*throw ()*/;

    /**
     * Sset a Guid from a String. If an empty string is passed, the GUID is set 
     * to null, but no error is returned
     */
    void fromString(const std::string& s) /*throw (LogicError)*/;
    
    /**
     * Create a string that represent the Guid
     */
    std::string str() const /*throw ()*/;

    /**
     * Compute the hash of the Guid
     */
    unsigned short hash() const /*throw()*/;   
   
    /**
     * Compute the cross sum of teh hash value
     */
    unsigned int crossSumHash() const /*throw()*/;   
 
private:
    /**
     * The ID
     */
    uuid_t m_id;
};
    
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_GUID_H_
