/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: URL Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Url.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_URL_H_
#define GLITE_DATA_AGENTS_URL_H_

#include <string>
#include <vector>

namespace glite {
namespace data  {
namespace agents{

/**
 * Url Class
 */
struct Url{
    /**
     * Schema property
     */
    std::string  schema;
    /**
     * Host Name property
     */
    std::string  host;
    /**
     * Port property
     */
    unsigned int port;
    /**
     * Path property
     */
    std::string  path;
    /**
     * Query String property
     */
    std::string  query;
    /**
     * Constructor
     */
    Url():port(0){}
    /**
     * Constructor. Call isGood in order to see if the URL is correctly parsed
     */
    Url(const std::string& str);
        
    /**
     * Return true if it's a valid url
     */
    bool isGood();
        
    /**
     * Create a string that represent the Url. In case the URL is not valid,
     * this method returns an empty string
     */
    std::string str() /*throw ()*/;
};
    
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_URL_H_
