/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: DAOContext Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DAOContext.h,v 1.5 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_DAOCONTEXT_H_
#define GLITE_DATA_AGENTS_DAO_DAOCONTEXT_H_

#include "glite/data/agents/dao/DAOException.h"
#include <boost/utility.hpp>

namespace glite {
namespace data  {
namespace agents{
namespace dao   {

/**
 * Base Class For DataAccessObject Context
 * @interface
 */
class DAOContext : boost::noncopyable {
public:

    /**
     * Destructor
     */ 
    virtual ~DAOContext(){}
        
    /**
     * Return the DN of the Agent Owning the Context
     */
    const std::string& agentDn() const /*throw ()*/ {return m_agentDn;}

    /**
     * Check if the Context is valid alive
     */
    virtual void check() /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Start a Transaction
     */
    virtual void start() /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Commit a Transaction
     */
    virtual void commit() /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Rollback a Transaction
     */
    virtual void rollback() /*throw ()*/ = 0;
    
    /**
     * Kill the connection asynchronously
     */
    virtual void kill() /*throw ()*/ = 0;
   
protected:

    /**
     * Constructor
     */ 
    explicit DAOContext(const std::string& dn = ""):m_agentDn(dn){}

private:
    /**
     * the DN of the Agent Owning the Context
     */
    const std::string& m_agentDn;

    /** @link dependency 
     * @label throws*/
    /*# DAOException lnkDAOException; */
};

} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_DAOCONTEXT_H_
