/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: DAOContextFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DAOContextFactory.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_DAOCONTEXTFACTORY_H_
#define GLITE_DATA_AGENTS_DAO_DAOCONTEXTFACTORY_H_

#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/dao/DAOException.h"
#include <boost/utility.hpp>

namespace glite {
namespace data  {
namespace agents{
namespace dao   {

/**
 * Base Class For DataAccessObject Factories
 * @interface
 */
class DAOContextFactory : boost::noncopyable {
public:

    /**
     * Creates a DataAccessObject Context
     */
    virtual DAOContext * create() /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Check if the the DB Schema is compatible with the requested one
     */
    virtual void checkSchema(      DAOContext&  ctx,
                             const std::string& table, 
                                   unsigned int major,
                                   unsigned int minor,
                                   unsigned int patch) /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Return the configured DAOContextFactory Instance. Please note that 
     * this method doesn't allocate the instance (it's not a singleton)
     * @return the configued DAOContextFactory 
     * @throws DAOLogicError in case of no DAOContextFactory is configured
     */
    static DAOContextFactory& instance() /*throw (DAOLogicError)*/; 
   
protected:

    /**
     * Constructor
     */ 
    DAOContextFactory();

    /**
     * Destructor
     */ 
    virtual ~DAOContextFactory();

private:

    /**
     * The registered instance
     */
    static DAOContextFactory * s_instance;
    
    /** @link dependency 
     * @label creates*/
    /*# DAOContext lnkDAOContext; */

    /** @link dependency 
     * @label throws*/
    /*# DAOException lnkDAOException; */
};

} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_DAOFACTORY_H_
