/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: DAOException Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DAOException.h,v 1.2 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_DAOEXCEPTION_H_
#define GLITE_DATA_AGENTS_DAO_DAOEXCEPTION_H_

#include "glite/data/agents/AgentExceptions.h"

namespace glite {
namespace data  {
namespace agents{
namespace dao   {
    
/**
 * The DAOException is raised when an error happens contacting the DataSource
 * @stereotype exception
 */
struct DAOException : public ExecutionError {
    /**
     * Constructor
     * @param reason [IN] the reason of the error
     */
    DAOException(const std::string& reason):ExecutionError(reason){}

    /**
     * Destructor
     */
    virtual ~DAOException() throw() {}
};

/**
 * The DAOException is raised when a logic error happens contacting the DataSource
 * @stereotype exception
 */
struct DAOLogicError : public LogicError {
    /**
     * Constructor
     * @param reason [IN] the reason of the error
     */
    DAOLogicError(const std::string& reason):LogicError(reason){}

    /**
     * Destructor
     */
    virtual ~DAOLogicError() throw() {}
};

} // end namespace dao
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_DAO_DAOEXCEPTION_H_
