/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: OracleDAO.h,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 *  Moved from org.glite.data.transfer-agents
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAO_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAO_H_

#include "glite/data/agents/glite-types.h"
#include "glite/data/agents/dao/dao-types.h"

#include <string>
#include <boost/utility.hpp>
#include <boost/scoped_ptr.hpp>
#include <occi.h>

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

class OracleDAOImpl;
class OracleDAOContext;

/**
 * Oracle Data Access Object Base class
 */
class OracleDAO : boost::noncopyable
{
public:
    /**
     * Return the DAO Context
     */
    OracleDAOContext& ctx();
    
    /**
     * Convert a Timestamp into a time value
     */
    time_t toTimeT(const ::oracle::occi::Timestamp& timestamp);

    /**
     * Convert a time value into a Timestamp 
     */
    ::oracle::occi::Timestamp toTimestamp(time_t t);

    /**
     * Convert a Number into a longlong
     */
    agents::longlong toLongLong(const ::oracle::occi::Number& number);

    /**
     * Convert a longlong into a Number
     */
    ::oracle::occi::Number toNumber(agents::longlong n);

    /**
     * Convert a Char into a boolean
     * @param str [IN] the value as string
     * @param defaultValue [IN] the value to use if the string is empty
     * @return boolean
     */
    bool toBoolean(const std::string& str,bool defaultValue) const;

    /**
     * Convert a Boolean into a String
     */
    const std::string& toBoolean(bool b) const;

    /**
     * Read a Clob into a String
     */
    void toString(::oracle::occi::Clob clob, std::string& str);
    
    /**
     * Read a string into a string. It basically returns the original string...
     */
    const std::string& toString(const std::string& str) const;

protected:
    OracleDAO(OracleDAOContext& ctx, const std::string& category);

protected:
    boost::scoped_ptr<OracleDAOImpl> m_impl;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAO_H_
