/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleDAOContext Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: OracleDAOContext.h,v 1.9 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOCONTEXT_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOCONTEXT_H_

#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/glite-types.h"
#include "occi.h"
#include <glite/data/config/service/Logger.hxx>

namespace oracle{
namespace occi{
    class Environment;
    class Connection;
}
}

namespace glite {
namespace data  {
namespace agents{
namespace dao   {
namespace oracle{

/**
 * Oracle DAO Context Class. A sort of wrapper for the connection object
 */
class OracleDAOContext : public DAOContext {
public:

    /**
     * Constructor
     */
    OracleDAOContext(class ::oracle::occi::Environment& env, const std::string& dn = "") throw ();

    /**
     * Destructor
     */
    virtual ~OracleDAOContext();

    // -------------------------------------------------------------------------
    // DAOContext Methods
    // -------------------------------------------------------------------------

    /**
     * validate the context
     */
    virtual void check() /*throw (DAOLogicError,DAOException)*/;

    /**
     * Start a Transaction
     */
    virtual void start() /*throw (DAOLogicError,DAOException)*/;

    /**
     * Commit a Transaction
     */
    virtual void commit() /*throw (DAOLogicError,DAOException)*/;

    /**
     * Rollback a Transaction
     */
    virtual void rollback() /*throw ()*/;

    /**
     * kill context
     */
    virtual void kill() /*throw ()*/;

    // -------------------------------------------------------------------------
    // Utility Methods
    // -------------------------------------------------------------------------
    
    /**
     * Convert a Timestamp into a time value
     */
    time_t toTimeT(const ::oracle::occi::Timestamp& timestamp) /*throw ()*/;

    /**
     * Convert a time value into a Timestamp 
     */
    ::oracle::occi::Timestamp toTimestamp(time_t t) /*throw ()*/;

    /**
     * Convert a Number into a longlong
     */
    longlong toLongLong(const ::oracle::occi::Number& number) /*throw ()*/;

    /**
     * Convert a longlong into a Number
     */
    ::oracle::occi::Number toNumber(longlong n) /*throw ()*/;

    /**
     * Convert a Char into a boolean
     */
    bool toBoolean(const std::string& str) const /*throw ()*/;

    /**
     * Convert a Boolean into a String
     */
    const std::string& toBoolean(bool b) const /*throw ()*/;

    /**
     * Read a Clob into a String
     */
    void toString(::oracle::occi::Clob clob, std::string& str) const /*throw (DAOException)*/;

    // -------------------------------------------------------------------------
    // Getter Methods
    // -------------------------------------------------------------------------

    /**
     * Get the Connection Pointer
     */
    class ::oracle::occi::Connection& connection() {return *m_connection;}
    
    /**
     * Get the Connection Pointer
     */
    class ::oracle::occi::Environment& env() const throw() {return m_env;}

    // -------------------------------------------------------------------------
    // Control Methods
    // -------------------------------------------------------------------------

    /**
     * Initialize the Connection
     */
    void init(const std::string  & user,
              const std::string  & password,
              const std::string  & connectString,
                    unsigned long  stmtCacheSize) /*throw (DAOLogicError)*/;

    /**
     * Destroy the connection
     */
    void dispose() /*throw ()*/;

    /**
     * Check the connection
     */
    bool isAlive() /*throw ()*/;

private:

    /**
     * Default Constructor. Not Implemented
     */
    OracleDAOContext();

private:
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
    
    /**
     * The Connection
     */
    class ::oracle::occi::Environment& m_env;
    
    /**
     * The Connection
     */
    class ::oracle::occi::Connection * m_connection;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOCONTEXT_H_
