/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: ResultSetPtr Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ResultSetPtr.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_RESULTSETPTR_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_RESULTSETPTR_H_

#include "occi.h"
#include <boost/utility.hpp>

namespace glite {
namespace data  {
namespace agents{
namespace dao   {
namespace oracle{

/**
 * Oracle ResultSet Smart Pointer. It will acquire the ownership of the 
 * ResultSet object and then call closeResultSet when destroyed (RAII)
 */
class ResultSetPtr : boost::noncopyable {
public:

    /**
     * Constructor
     */
    ResultSetPtr(::oracle::occi::Statement&  stmt,
                 ::oracle::occi::ResultSet * rs) throw ():m_stmt(stmt),m_rs(rs){}

    /**
     * Destructor
     */
    ~ResultSetPtr() throw () {
        try{
            if(0 != m_rs){
                m_stmt.closeResultSet(m_rs);
            }
        }catch(...){}
    }

    /**
     * Operator*
     */
    ::oracle::occi::ResultSet& operator*() const {return *m_rs;}
    
    /**
     * Operator->
     */
    ::oracle::occi::ResultSet* operator->() const throw() {return m_rs;}
    
    /**
     * Get the Statement Pointer
     */
    ::oracle::occi::ResultSet* get() const throw() {return m_rs;}
    
    /**
     * Get the Statement Pointer
     */
    ::oracle::occi::ResultSet* release() throw() {
        ::oracle::occi::ResultSet* rs = m_rs;
        m_rs = 0;
        return rs;
    }
    
private:
    /**
     * Default Constructor. Not Implemented
     */
    ResultSetPtr();

private:
    /**
     * The Statement
     */
    ::oracle::occi::Statement&  m_stmt;
    
    /**
     * The ResultSet 
     */
    ::oracle::occi::ResultSet * m_rs;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_ORACLE_RESULTSETPTR_H_
