/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: StatementPtr Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: StatementPtr.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_STATEMENTPTR_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_STATEMENTPTR_H_

#include "occi.h"
#include <boost/utility.hpp>

namespace glite {
namespace data  {
namespace agents{
namespace dao   {
namespace oracle{

/**
 * Oracle Statement Smart Pointer. It will acquire the ownership of the 
 * Statement object and then call terminateStatement when destroyed (RAII)
 */
class StatementPtr : boost::noncopyable {
public:

    /**
     * Constructor
     */
    StatementPtr(::oracle::occi::Connection& conn, 
                 ::oracle::occi::Statement * stmt) throw (): m_connection(conn),m_stmt(stmt){}

    /**
     * Destructor
     */
    ~StatementPtr() throw () {
        try{
            if(0 != m_stmt){
                m_connection.terminateStatement(m_stmt);
            }
        }catch(...){}
    }

    /**
     * Operator*
     */
    ::oracle::occi::Statement& operator*() const {return *m_stmt;}
    
    /**
     * Operator->
     */
    ::oracle::occi::Statement* operator->() const throw() {return m_stmt;}
    
    /**
     * Get the Statement Pointer
     */
    ::oracle::occi::Statement* get() const throw() {return m_stmt;}
    
    /**
     * Release the Statement Pointer
     */
    ::oracle::occi::Statement* release() throw() {
        ::oracle::occi::Statement* stmt = m_stmt;
        m_stmt = 0;
        return stmt;
    }
    
private:
    /**
     * Default Constructor. Not Implemented
     */
    StatementPtr();

private:
    /**
     * The Connection
     */
    ::oracle::occi::Connection& m_connection;
    
    /**
     * The Statement
     */
    ::oracle::occi::Statement * m_stmt;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite


#endif //GLITE_DATA_AGENTS_DAO_ORACLE_STATEMENTPTR_H_
