/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *  GLite Data Transfer Agents: Base class for Oracle Statements
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 *  Version info: $Id: Statements.h,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_STATEMENTS_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_STATEMENTS_H_

#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/model/Property.h"

#include <string>
#include <vector>
#include <boost/utility.hpp>
#include <sstream>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

class Statements{
protected:
    
    /**
     * constructor
     * @param logger_instance [IN] logger initialization parameter
     */
    Statements(const char* logger_instance);

    /**
     * destructor
     * (not virtual since protected)
     */
    ~Statements(){}

    /**
     * create_statement_string function pointer typedef
     */
    typedef const char* (*create_statement_string) ();
    
    /**
     * create_statement
     * helper function for statements creations
     * @param ctx [IN] the oracle DAO context
     * @param tag [IN] the statement cache tag
     * @param ptr [IN] pointer to function that returns the statement string
     * @param name [IN] statement name (for errors logging)
     */
    ::oracle::occi::Statement* create_statement(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const char*                             tag,
        create_statement_string                 ptr,
        const char*                             name);
    
    glite::config::Logger m_logger;
};

/**
 * add_property_update
 * append to stream part of update statement
 * @param query_stmt [IN/OUT]
 * @param count [IN/OUT]
 * @param start_count [IN]
 * @param property [IN]
 * @param property_name [IN]
 */
template<typename T>
void add_property_update(
    std::stringstream&          query_stmt,
    unsigned int&               count,
    const unsigned int&         start_count,
    const model::Property<T>&   property,
    const char*                 property_name){
        
    if (property.isSet()){
        if (count != start_count){
            query_stmt << ",";
        }
        query_stmt << " " << property_name << " = :" << count;
        count++;
    }        
}

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif // GLITE_DATA_AGENTS_DAO_ORACLE_STATEMENTS_H_
