/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: PatternList Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PatternList.h,v 1.4 2007/03/15 09:07:14 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_EXTRA_PATTERN_LIST_H_
#define GLITE_DATA_AGENTS_EXTRA_PATTERN_LIST_H_

#include <string>
#include <vector>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/regex.hpp>

namespace glite {
namespace data  {
namespace agents{

/**
 * PatternList class: used to evaluate a set of pattern
 */
class PatternList: boost::noncopyable{
public:
	
	/**
     * maximum length of string to match
     */
	static const std::string::size_type MaxMatchLen;

    /**
     * Default Constructor
     */
    PatternList(){}

    /**
     * Constructor
     */
    PatternList(const char * const p[], unsigned int s);
    
    /**
     * Constructor
     */
    explicit PatternList(const std::vector<std::string>& p);

    /**
     * add a pattern
     */
    void add(const char * pattern) /*throw (boost::bad_expression)*/;

    /**
     * Return true if the string matches one of the patterns
     */
    bool match(const std::string& str, bool partial_match = false);

private:
    /*
     * The pattern list
     */
    std::vector<boost::shared_ptr<boost::regex> > m_patterns;
};

} // end namespace glite
} // end namespace data
} // end namespace agents

#endif //GLITE_DATA_AGENTS_EXTRA_PATTERN_LIST_H_
