/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Extra Utility functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: utility.h,v 1.7 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_EXTRA_UTILITY_H_
#define GLITE_DATA_AGENTS_EXTRA_UTILITY_H_

#include <string>
#include <algorithm>
#include <ctype.h>

namespace glite {
namespace data  {
namespace agents{
   
/*
 * eq_nocase
 *
 * Character Comparison that ignore the case
 */
inline bool eq_nocase(char c1, char c2){ 
    return tolower(c1) == tolower(c2);
}

/**
 * Compare the first element of a std::pair
 */
struct compare_first: std::unary_function<const std::pair<std::string,std::string>&, bool> {
    /**
     * Constructor
     */
    compare_first(const std::string& s):m_str(s){}
    /**
     * Call Operator
     */
    bool operator()(const std::pair<std::string,std::string>& p) const {
        return ( p.first == m_str )?true:false;
    }
private:
    const std::string& m_str;
};

/**
 * Get the name of the site where the SURL is located
 */
std::string get_site_for_surl(const std::string& surl, const std::string& vo_name, bool enable_unknown) /*throw (LogicError, RuntimeError)*/;

/**
 * Convert a string representing a size (in bytes) to an integer. Examples of the allowed formats are:
 *   122
 *   100 KB
 *   1.2 MB (or without the space 1.2MB)
 *   .5 GB
 */
unsigned long str_to_size(std::string str) /*throw (LogicError)*/;

/**
 * Return the hash of a string 
 */
unsigned long hash_string(const std::string& str) throw ();
   
/**
 * Get the parent Directory name
 */
std::string get_dirname(const std::string& dir_path) throw ();

} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_EXTRA_UTILITY_H_
