/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Python Helper Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PyHelper.h,v 1.3 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_PYTHON_PYHELPER_H_
#define GLITE_DATA_AGENTS_PYTHON_PYHELPER_H_

#include <boost/python.hpp>
#include <string>
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace python{

/**
 * Python Helper Class
 */
class PyHelper {
public:

    /**
     * Return the SDConfig singleton instance
     * @return the SDConfig singleton instance
     */
    static PyHelper& instance() /*throw ()*/; 

    /**
     * extend PYTHONPATH by adding additional path
     */
    void setPath(const std::string& p) /*throw ()*/; 

    /**
     * Import a module
     */
    boost::python::object import(const std::string& module_name) /*throw (PythonException)*/;

    /**
     * Return a function object
     */
    boost::python::object getFunction(boost::python::object& module, const std::string& function_name) /*throw (PythonException)*/;

    /**
     * Translate Python Error in type, value
     */
    std::pair<std::string, std::string> lastError() /*throw ()*/;
    
protected:
    
    /**
     * Initialize the Python Interpreter
     */
    void init() /*throw (PythonException)*/;

    /**
     * Finalize the Python Interpreter
     */
    void fini() /*throw ()*/;

private:
    
    /*
     * Declare factory methods as friends
     */
    friend class PythonConfig;
        
    /**
     * Constructor
     */
    PyHelper();
        
    /**
     * Destructor
     */
    ~PyHelper();

private:
        
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
        
    /**
     * The PythonObject corresponding to the main module
     */
    boost::python::object m_mainModule;
    
    /**
     * The PythonObject corresponding to the dictionary of the main module
     */
    boost::python::object m_global;
   
};

} // end namespace python
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_PYTHON_PYHELPER_H_
