/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: ServiceDiscovery Types
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDTypes.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SDTYPES_H_
#define GLITE_DATA_AGENTS_SD_SDTYPES_H_

#include <string>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {
    
/**
 * SD Schema Version
 */
extern const char * const SD_SCHEMA_VERSION;
    
/**
 * The Service Class
 */
struct Service {
    /** 
     * The name of the service 
     */
    std::string name;
    /** 
     * The type of service 
     */
    std::string type;
    /** 
     * The endpoint used to contact the service
     */
    std::string endpoint;
    /** 
     * The service version
     */
    std::string version;
    /** 
     * The service site
     */
    std::string site;
    /** 
     * The service hostname
     */
    std::string hostname;
    /** 
     * The service port
     */
    unsigned int port;

    /** 
     * Default Constructor
     */
    Service():port(0){}
        
    /** 
     * Constructor
     */
    Service(const std::string& n,  // name
        const std::string& t,      // type
        const std::string& e,      // endpoint
        const std::string& v,      // version 
        const std::string& s,      // site
        const std::string& h,      // hostname
        int p = 0):                // port
        name(n),type(t),endpoint(e),version(v),site(s),hostname(h),port(p){}
};

/**
 * VirtualOrganization Class
 */
struct VirtualOrganization
{
    /**
     * Virtual Organization Name
     */
    std::string name;
    /** 
     * Constructor                                                                                                                                                         
     */
    explicit VirtualOrganization(const std::string& n):name(n){}
};
  
} // end namespace sd
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_SD_SDTYPES_H_
