/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: SD Predicates Classes
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: sd-predicates.h,v 1.4 2007/11/05 11:31:20 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SD_PREDICATES_H_
#define GLITE_DATA_AGENTS_SD_SD_PREDICATES_H_

#include "glite/data/agents/sd/sd-utility.h"
#include "glite/data/agents/sd/SDConfig.h"
#include <string>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {

/**
 * Service Selection predicate: Select the Myproxy Server which role is either
 * renewer or unset
 */
class SelectMyProxyForRenewal: public SelectPred {
public:	
    /**
     * Contructor
     */
    explicit SelectMyProxyForRenewal(const std::string& vo_name = ""):m_voName(vo_name){}
    
    virtual const char * type(){return "SelectMyProxyForRenewal";}
    
protected:
    /**
     * Call Operator
     */
    virtual const Service* select(const std::vector<const Service*>& services) throw (){
        const Service * service = 0;
        try{
            std::vector<const Service*>::const_iterator it;
            for(it = services.begin(); it != services.end(); ++it){
                const Service * s = (*it);
                // Get MyProxy Role
                std::string role = get_property(*s,
                    SDConfig::instance().myproxyFtsModeProperty(),m_voName);
                // Check MyProxy Role
                if((true == role.empty()) || 
                   (role == SDConfig::instance().myproxyFtsModeRenewer())){
                    service = s;
                    break;
                }
            }
        } catch(...){
            // Ignore errors
        }
        // Return the found service. In case no MyProxy servers match the 
        // criteria, return 0
        return service;
     }
private:
    /**
     * The name of the Vo used in the predicate
     */
    std::string m_voName;
};

/**
 * Service Selection predicate: Select the Myproxy Server which role is
 * retriver
 */
class SelectMyProxyForRetrieval: public SelectPred {
public:
    /**
     * Contructor
     */
    explicit SelectMyProxyForRetrieval(const std::string& vo_name = ""):m_voName(vo_name){}

    virtual const char * type(){return "SelectMyProxyForRenewal";}
    
protected:
    
    /**
     * Call Operator
     */
    virtual const Service* select(const std::vector<const Service*>& services) throw (){
        const Service * service = 0;
        try{
            std::vector<const Service*>::const_iterator it;
            for(it = services.begin(); it != services.end(); ++it){
                const Service * s = (*it);
                // Get MyProxy Role
                std::string role = get_property(*s,
                    SDConfig::instance().myproxyFtsModeProperty(),m_voName);
                // Check MyProxy Role
                if(role == SDConfig::instance().myproxyFtsModeRetriever()){
                    service = s;
                    break;
                }
            }
        } catch(...){
            // Ignore errors
        }
        // Return the found service. In case no MyProxy servers match the 
        // criteria, return the first
        if(0 == service){
            service = *(services.begin());
        }
        return service;
     }
private:
    /**
     * The name of the Vo used in the predicate
     */
    std::string m_voName;
};

    
} // End namespace sd
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_SD_SD_PREDICATES_H_
