/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SURLHelpers Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: surl-utility.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SURL_UTILITY_H_
#define GLITE_DATA_AGENTS_SURL_UTILITY_H_

#include <string>

namespace glite {
namespace data  {
namespace agents{
 
/**
 * Translate the SURL into its fully qualified form:
 *  srm://<host>:<port>/srm/managerv1?SFN=<file_path> 
 */
std::string fully_qualify_surl(const std::string& surl,const std::string& srm_endpoint) /*throw (LogicError)*/;

/**
 * Translate the SURL into its compact form:
 *  srm://<host>[:<port>]/<file_path>
 */
std::string compact_surl(const std::string& surl,const std::string& srm_endpoint,bool with_port) /*throw (LogicError)*/;

/**
 * Extract the file path from a surl
 */
std::string path_from_surl(const std::string& surl) /*throw (LogicError)*/;

} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_SURL_UTILITY_H_
