/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Url Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Url.cpp,v 1.4 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/Url.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/AgentExceptions.h"

#include <glite/data/config/service/Logger.hxx>
using namespace glite::data::agents;


namespace {
const char * const COMPONENT_NAME = "url-utility";
}

/*
 * Url
 *
 * Constructor
 */
Url::Url(const std::string& str):port(0){
    try{
        parse_url(str,this->host,this->port,this->schema,this->path,this->query);
    } catch(const LogicError& exc){
        _log_error(COMPONENT_NAME,"Failed to parse Url. Reason is " << exc.what());
    }
}
        
/*
 * isGood
 *
 * Return true if it's a valid url
 */
bool Url::isGood(){
    if(false == this->schema.empty()){
        if((false == this->host.empty()) || (false == this->path.empty())){
            return true;
        }
    }
    return false;
}
    
/*
 * str
 *
 * Create a String
 */
std::string Url::str(){
    std::string url;
    try{
        url = create_url(schema,host,port,path,query,false);
    } catch(const LogicError& exc){
        _log_error(COMPONENT_NAME,"Failed to parse Url. Reason is " << exc.what());
    }
    return url;
}
