/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: MyProxyClient Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: MyProxyClient.h,v 1.4 2007/01/24 12:08:22 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_COMMON_CRED_MYPROXYCLIENT_H_
#define GLITE_DATA_AGENTS_COMMON_CRED_MYPROXYCLIENT_H_

#include "glite/data/agents/cred/CredService.h"
#include <boost/utility.hpp>

namespace glite  {
namespace data   {
namespace agents {
namespace cred   {
namespace myproxy{

/**
 * MyProxy Client
 */
class MyProxyClient : public CredService, boost::noncopyable{
public:
    /**
     * Constructor
     */
    MyProxyClient(const class MyProxyClientFactory& f, const std::string& server);
        
    /**
     * Destructor
     */
    virtual ~MyProxyClient();

protected:

    /**
     * Generate a Name for the Certificate Proxy File
     */
    virtual std::string getFileName(const std::string& userDn, const std::string& passphrase) /* throw(LogicError) */; 

    /**
     * Get the Cerificate From the MyProxy Server
     */
    virtual void getNewCertificate(const std::string& userDn, const std::string& passphrase, const std::string& filename) /*throw (MyProxyClientException)*/;

    /**
     * Returns the validity time that the cached copy of the certificate should 
     * have. 
     */
    virtual unsigned long minValidityTime() /* throw() */;

private:
    
    /**
     * URL Encode a String
     */
    std::string urlEncode(const std::string& str);

    /*
     * Parse the server endpoint. It can be in the format: 
     *      myproxy://<host>[:<port>]
     *      myproxy:<host>[:<port>]
     *      <host>[:<port>]
     */
    void parseServerEndpoint(std::string& server_host,unsigned int& server_port);

private:
    
    /**
     * Default Constructor. Not Implemented
     */
    MyProxyClient();

private:
    /**
     * The factory that creates the object
     */
    const class MyProxyClientFactory& m_factory;

    /**
     * The user specified MyProxy server to use
     */
    std::string m_server;

   /** @link dependency 
     * @label throws*/
    /*# MyProxyClientException lnkMyProxyClientException; */
};

} // end namespace myproxy
} // end namespace cred
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_COMMON_CRED_MYPROXYCLIENT_H_
