/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: DAOContextFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DAOContextFactory.cpp,v 1.2 2007/01/24 12:08:22 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/agents/dao/DAOException.h"

using namespace glite::data::agents::dao;

/*
 * Initialize Static Attributes
 */
DAOContextFactory * DAOContextFactory::s_instance = 0;

/*
 * getInstance
 *
 * Return the registered Instance
 */
DAOContextFactory& DAOContextFactory::instance() /*throw (DAOLogicError)*/ {
    if(0 == s_instance){
        throw DAOLogicError("No DAOContextFactory configured");
    }
    return (*s_instance);
}

/*
 * DAOContextFactory
 *
 * Constructor
 */
DAOContextFactory::DAOContextFactory(){
    // Register the instance
    if(0 == s_instance){
        s_instance = this;
    }
}
                                                                                                                          
/*
 * ~DAOContextFactory
 *
 * Destructor
 */
DAOContextFactory::~DAOContextFactory(){
    if(this != s_instance){
        s_instance = 0;
    }
}
