/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *  GLite Data Transfer Agents: OracleDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: OracleDAO.cpp,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAO.h"

#include "OracleDAOImpl.h"

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

OracleDAOContext& OracleDAO::ctx()
{
    return m_impl->m_ctx;
}
    
time_t OracleDAO::toTimeT(const ::oracle::occi::Timestamp& timestamp)
{
    return m_impl->m_ctx.toTimeT(timestamp);
}

::oracle::occi::Timestamp OracleDAO::toTimestamp(time_t t)
{
    return m_impl->m_ctx.toTimestamp(t);
}

agents::longlong OracleDAO::toLongLong(const ::oracle::occi::Number& number)
{
    return m_impl->m_ctx.toLongLong(number);
}

::oracle::occi::Number OracleDAO::toNumber(agents::longlong n)
{
    return m_impl->m_ctx.toNumber(n);
}

bool OracleDAO::toBoolean(const std::string& str, bool defaultValue) const
{
    if (str.empty()){
        return defaultValue;
    } else {
        return m_impl->m_ctx.toBoolean(str);
    }
}

const std::string& OracleDAO::toBoolean(bool b) const
{
    return m_impl->m_ctx.toBoolean(b);
}

void OracleDAO::toString(::oracle::occi::Clob clob, std::string& str)
{
    m_impl->m_ctx.toString(clob,str);
}

const std::string& OracleDAO::toString(const std::string& str) const
{
    return str;
}

OracleDAO::OracleDAO(OracleDAOContext& ctx, const std::string& category)
    : m_impl(new OracleDAOImpl(ctx, category))
{
    // EMPTY
}

} // End namespace oracle
} // End namespace dao
} // End namespace agenta
} // End namespace data
} // End namespace glite
