/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleDAOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: OracleDAOConfig.h,v 1.5 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOCONFIG_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOCONFIG_H_

#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "occi.h"
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace dao   {
namespace oracle{

/**
 * DataAccessObject Configuration For Oracle
 */
class OracleDAOConfig : public DAOContextFactory{
public:

    /**
     * Return the singleton instance
     * @return the singleton instance
     */
    static OracleDAOConfig& instance() /*throw ()*/; 

    /**
     * Initialize a Oracle DAO Context
     */
    OracleDAOContext * createContext() /*throw (DAOLogicError, DAOException)*/;

    /**
     * Check that an Oracle Context is still valid, otherwise revalidate it
     */
    void checkContext(OracleDAOContext& ctx) /*throw (DAOLogicError, DAOException)*/;

    // -------------------------------------------------------------------------
    // Implements DAOContextFactory Method
    // -------------------------------------------------------------------------

    /**
     * Initialize a Oracle DAO Context
     */
    virtual DAOContext * create() /*throw (DAOLogicError, DAOException)*/ {
        return createContext();
    }

    /**
     * Check if the the DB Schema is compatible with the requested one
     */
    virtual void checkSchema(DAOContext& ctx, const std::string& table, unsigned int major,unsigned int minor,unsigned int patch) /*throw (DAOLogicError, DAOException)*/;

protected:

    /*
     * Declare Friend Class
     */
    friend class OracleDAOComponent;

    /**
     * Initialize the Component
     */
    void init() /*throw (DAOLogicError)*/;

    /**
     * Finailize the Component
     */
    void fini();

    /**
     * Set DB User Property
     */
    void user(const std::string& u) {m_user = u;}

    /**
     * DB Password Property
     */
    void password(const std::string& p) {m_password = p;}
    
    /**
     * Connect String Property
     */
    void connectString(const std::string& c) {m_connectString = c;}

    /**
     * Statement Cache Size Property
     */
    void stmtCacheSize(unsigned int c) {m_stmtCacheSize = c;}
    
    /**
     * Statement Cache Size Property
     */
    void threadingModel(const std::string& tm) {m_threadingModel = tm;}
    
private:

    /**
     * Constructor
     */
    OracleDAOConfig();

    /**
     * Destructor
     */
    virtual ~OracleDAOConfig();
   
private:
     
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
    
    /**
     * DB User Property
     */
    std::string m_user;

    /**
     * DB Password Property
     */
    std::string m_password;
    
    /**
     * Connect String Property
     */
    std::string m_connectString;

    /**
     * Statement Cache Size Property
     */
    unsigned int m_stmtCacheSize;
    
    /**
     * Environment Threading Model Property
     */
    std::string m_threadingModel;

    /**
     * OCCI Environmanet
     */
    ::oracle::occi::Environment * m_environment;
    
    /**
     * The AgentDN
     */
    std::string m_agentDn;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOCONFIG_H_
