/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: OracleDAOImpl.h,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 *  Moved from org.glite.data.transfer-agents
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOIMPL_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOIMPL_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/agents/glite-types.h"
#include "glite/data/agents/dao/dao-types.h"

#include "glite/data/agents/dao/oracle/OracleDAOContext.h"

#include <glite/data/config/service/Logger.hxx>

#include <string>
#include <boost/utility.hpp>
#include <occi.h>

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

/** Implementation details of the OracleDAO class */
struct OracleDAOImpl : boost::noncopyable
{
    OracleDAOImpl(OracleDAOContext& ctx, const std::string& category)
        : m_logger(category),
          m_ctx(ctx)
    {
        // EMPTY
    }

    // m_log_* functions have been defined as macros. Those macros imply that
    // the user of the functions have a m_logger member, and calls functions
    // on that. Therefore, we need the wrappers below. Should have done it
    // better...
    log4cpp::CategoryStream log_trace(const std::string& message)
    {
        m_log_trace(message);
        return m_logger.logger().debugStream();
    }

    log4cpp::CategoryStream log_error(const std::string& message)
    {
        m_log_error(message);
        return m_logger.logger().errorStream();
    }

    glite::config::Logger m_logger;
    OracleDAOContext& m_ctx;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_ORACLE_ORACLEDAOIMPL_H_
