/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *  GLite Data Transfer Agents: OracleJobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: OracleSeDAO.cpp,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 */
#include "OracleSeDAO.h"
#include "OracleDAOImpl.h"
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>
#include <glite/data/agents/dao/DAOException.h>

#include <memory>
#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

OracleSeDAO::~OracleSeDAO()
{
    // EMPTY
}

OracleSeDAO::OracleSeDAO(OracleDAOContext& ctx, const std::string& category)
    : OracleDAO(ctx, category)
{
    // EMPTY
}

model::Se * OracleSeDAO::get(::oracle::occi::Statement& stmt)
{
    m_impl->log_trace("Calling OracleSeDAO::get");
    std::auto_ptr<model::Se> se;

    try{
        // Execute Query
        ResultSetPtr rs(stmt, stmt.executeQuery());

        if(0 != rs->next()) {
            se.reset(new model::Se(
                toString(rs->getString(1)),
                toString(rs->getString(2)),
                toString(rs->getString(3)),
                toString(rs->getString(4)),
                toString(rs->getString(5)),
                toString(rs->getString(6)),
                toString(rs->getString(7))
            ));
        } else {
            throw DAOException("No SE found for the given name");
        }
    } catch(const SQLException& exc){
        m_impl->log_error("Got SQLException in Executing Get: [0x") << exc.getErrorCode() << "] " << exc.getMessage();
        throw DAOException("Failed to get SE - SQL Exception");
    } catch(const DAOException& exc){
        m_impl->log_error("Got DAOException in Executing Get: ") << exc.what();
        throw;
    }

    m_impl->log_trace("OracleSeDAO::get Called");
    return se.release();
}

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite
