/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *  Background: https://savannah.cern.ch/bugs/?37531
 *
 *  Authors: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)*
 *
 *  Version info: $Id: SeStatements.cpp,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 */
#include "SeStatements.h"
//#include "dao-utility.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"

#include <sstream>
//
//using namespace glite::data::transfer::agent;
//using namespace glite::data::transfer::agent::dao;
//using namespace glite::data::transfer::agent::dao::oracle;
//using glite::data::agents::dao::DAOLogicError;
//using glite::data::agents::dao::DAOException;
//using glite::data::agents::dao::oracle::OracleDAOContext;
//using glite::data::agents::dao::oracle::StatementPtr;
//
//using namespace ::oracle::occi;

namespace {
const char * const ORACLE_JOB_STATEMENTS_NAME = "transfer-agent-dao-oracle-job-stmt";

const char * const GET_STATEMENT_VO_TAG             = "t_job_get_vo";
const char * const GET_FULL_SEARCH_STATEMENT_VO_TAG = "t_job_get_full_search_vo";
const char * const GET_STATEMENT_CH_TAG             = "t_job_get_ch";
const char * const GET_FULL_SEARCH_STATEMENT_CH_TAG = "t_job_get_full_search_ch";
const char * const GET_LAST_STATEMENT_CRED_TAG      = "t_job_get_last_cred";
const char * const UPDATE_STATEMENT_VO_TAG          = "t_job_update_vo";
const char * const UPDATE_STATEMENT_CH_TAG          = "t_job_update_ch";
const char * const GET_SUBMITTED_STATEMENT_VO_TAG   = "t_job_get_submitted_vo";
const char * const GET_PENDING_STATEMENT_VO_TAG     = "t_job_get_pending_vo";
const char * const GET_DONE_STATEMENT_VO_TAG        = "t_job_get_done_vo";
const char * const GET_FINISHING_STATEMENT_VO_TAG   = "t_job_get_finishing_vo";
const char * const GET_FILES_STATE_STATEMENT_VO_TAG = "t_job_get_files_state_vo";
const char * const GET_FILES_STATE_STATEMENT_CH_TAG = "t_job_get_files_state_ch";
const char * const GET_JOBS_TO_FIX_CH_TAG           = "t_job_get_to_fix_ch";
}

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

SeStatements& SeStatements::instance()
{
    static SeStatements singleton;
    return singleton;
}

SeStatements::SeStatements()
    : Statements(StatementName_().c_str())
{
    // EMPTY
}

SeStatements::~SeStatements()
{
    // EMPTY
}

const std::string& SeStatements::StatementName_()
{
    static const std::string s("transfer-agent-dao-oracle-se-stmt");
    return s;
}

const std::string& SeStatements::StatementTag_()
{
    static const std::string s("t_se_get");
    return s;
}

const char* SeStatements::SelectStatement_()
{
	static const std::string statement =
		"SELECT * FROM t_se "
	    "WHERE t_se.name = :1 ";

	return statement.c_str();
}

::oracle::occi::Statement* SeStatements::createGet(OracleDAOContext& ctx)
{
    return create_statement(ctx, StatementTag_().c_str(), SeStatements::SelectStatement_, "Get Full Search");
}

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

