/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *  Background: https://savannah.cern.ch/bugs/?37531
 *
 *  Authors: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 *
 *  Version info: $Id: SeStatements.h,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 */
#ifndef GLITE_DATA_AGENTS_DAO_ORACLE_SESTATEMENTS_H_
#define GLITE_DATA_AGENTS_DAO_ORACLE_SESTATEMENTS_H_

#include "glite/data/agents/model/Se.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/dao/oracle/Statements.h"

#include <boost/utility.hpp>
#include <string>

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

class TestSeStatements;

/**
 * Oracle Data Access Object For the model::Se class
 */
class SeStatements : boost::noncopyable, public Statements
{
    friend class TestSeStatements;
public:
    static SeStatements& instance();
        
    /** Return the statement for the SeDAO::get */
    ::oracle::occi::Statement* createGet(agents::dao::oracle::OracleDAOContext& ctx);

private:
    SeStatements();
    virtual ~SeStatements();

    static const std::string& StatementName_();
    static const std::string& StatementTag_();
    static const char* SelectStatement_();
};


} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_ORACLE_SESTATEMENTS_H_
