/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Define the SD Class Wrapper Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDModule.h,v 1.3 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_PYTHON_SD_MODULE_H_
#define GLITE_DATA_AGENTS_PYTHON_SD_MODULE_H_

#include "glite/data/agents/sd/SDTypes.h"

#include <boost/python.hpp>
#include <boost/shared_ptr.hpp>

/*
 * Forward Declaration of module name
 */
#ifdef __cplusplus 
extern "C" {
#endif //__cplusplus
void init_sd(void);
#ifdef __cplusplus 
}
#endif //__cplusplus

namespace glite {
namespace data  {
namespace agents{
namespace python{

/*
 * Python Module Name
 */    
extern const char * const SD_MODULE_NAME;

/*
 * get_sd_schema_version
 *
 * Return sd Schema Version
 */
const char * get_sd_schema_version();

/**
 * Return a Service Object
 */
boost::shared_ptr<sd::Service> get_sd_service(const std::string& source, const std::string& type, const std::string& vo_name);
    
/**
 * Return a Service Object by type
 */
boost::shared_ptr<sd::Service> get_sd_service_by_type(const std::string& type, const std::string& vo_name);

/**
 * Return an associated Service Object of the given type
 */
boost::shared_ptr<sd::Service> get_sd_associated_service(const std::string& assoc_srv_host,const std::string& assoc_srv_type, const std::string& srv_type,const std::string& vo_name);

/**
 * Return an the site name for a given Source (Host/Site)
 */
std::string get_sd_site_name(const std::string& source, const std::string& type, const std::string& vo_name);
 
} // end namespace python
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_PYTHON_SD_MODULE_H_
