/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Define the Utils Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: UtilsModule.h,v 1.5 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_PYTHON_UTILS_MODULE_H_
#define GLITE_DATA_AGENTS_PYTHON_UTILS_MODULE_H_

#include <boost/python.hpp>

#include <glite/data/config/service/Logger.hxx>

/*
 * Forward Declaration of module name
 */
#ifdef __cplusplus
extern "C" {
#endif //__cplusplus
void init_utils(void);
#ifdef __cplusplus
}
#endif //__cplusplus

namespace glite {
namespace data  {
namespace agents{
namespace python{

/*
 * Python Module Name
 */    
extern const char * const UTILS_MODULE_NAME;

/*
 * log_entry
 *
 * Insert a log entry
 */
void log_entry(const char * component,log4cpp::Priority::PriorityLevel p, const char * msg);

/*
 * log_debug
 *
 * Insert a debug log entry
 */
void log_debug(const char * component, const char * msg);

/*
 * log_info
 *
 * Insert a info log entry
 */
void log_info(const char * component, const char * msg);

/*
 * log_warn
 *
 * Insert a warn log entry
 */
void log_warn(const char * component, const char * msg);

/*
 * log_error
 *
 * Insert a error log entry
 */
void log_error(const char * component, const char * msg);

/**
 * Get the name of the file containing the Proxy Certificate for the given job
 */
std::string get_user_proxy_cert(const std::string& user_dn, const std::string& user_cred, const std::string& endpoint = "", const std::string& cred_type = "");

} // end namespace python
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_PYTHON_UTILS_MODULE_H_
