/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: SDConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDConfig.cpp,v 1.6 2007/01/24 12:08:23 badino Exp $ 
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/SDCache.h"

using namespace glite::data::agents::sd;

/*
 * instance
 */
const char * const glite::data::agents::sd::SD_SCHEMA_VERSION = "1.0.0";

namespace{
// Properties Default  Values
const char * const DEFAULT_SRM_SERVICE_TYPE                 = "SRM";
const char * const DEFAULT_GRIDFTP_SERVICE_TYPE             = "gsiftp";
const char * const LEGACY_GRIDFTP_SERVICE_TYPE              = "GridFTP";
const char * const DEFAULT_MYPROXY_SERVICE_TYPE             = "MyProxy";
const char * const DEFAULT_FILETRANSFER_SERVICE_TYPE        = "org.glite.FileTransfer";
const char * const DEFAULT_FILETRANSFERSTATS_SERVICE_TYPE   = "org.glite.FileTransferStats";
const char * const DEFAULT_CHANNELMANAGEMENT_SERVICE_TYPE   = "org.glite.ChannelManagement";
const char * const DEFAULT_CHANNELAGENT_SERVICE_TYPE        = "org.glite.ChannelAgent";
const char * const DEFAULT_BDII_SERVICE_TYPE                = "BDII";
const char * const DEFAULT_VOMS_SERVICE_TYPE                = "org.glite.voms";
const char * const DEFAULT_SEINDEX_SERVICE_TYPE             = "org.glite.SEIndex";
const char * const DEFAULT_FIREMAN_SERVICE_TYPE             = "org.glite.Fireman";

const char * const DEFAULT_SEPATH_PROPERTY                  = "SEMountPoint";
const char * const DEFAULT_CHANNELAGENTSOURCE_PROPERTY      = "source";
const char * const DEFAULT_CHANNELAGENTDESTINATION_PROPERTY = "destination";
const char * const DEFAULT_MYPROXYFTSMODE_PROPERTY          = "FtsMode";
const char * const DEFAULT_MYPROXYFTSMODE_RETRIEVER         = "retriever";
const char * const DEFAULT_MYPROXYFTSMODE_RENEWER           = "renewer";
}

/*
 * instance
 *
 * Return the singleton Instance
 */
SDConfig& SDConfig::instance() {
    static SDConfig the_instance;
    return the_instance;
}

/*
 * SDConfig
 *
 * Constructor
 */
SDConfig::SDConfig():
    m_srmType(DEFAULT_SRM_SERVICE_TYPE),
    m_gridftpType(DEFAULT_GRIDFTP_SERVICE_TYPE),
    m_legacyGridftpType(LEGACY_GRIDFTP_SERVICE_TYPE),
    m_myproxyType(DEFAULT_MYPROXY_SERVICE_TYPE),
    m_fileTransferType(DEFAULT_FILETRANSFER_SERVICE_TYPE),
    m_fileTransferStatsType(DEFAULT_FILETRANSFERSTATS_SERVICE_TYPE),
    m_channelManagementType(DEFAULT_CHANNELMANAGEMENT_SERVICE_TYPE),
    m_channelAgentType(DEFAULT_CHANNELAGENT_SERVICE_TYPE),
    m_bdiiType(DEFAULT_BDII_SERVICE_TYPE),
    m_vomsType(DEFAULT_VOMS_SERVICE_TYPE),
    m_seIndexType(DEFAULT_SEINDEX_SERVICE_TYPE),
    m_firemanType(DEFAULT_FIREMAN_SERVICE_TYPE),
    m_sePathProperty(DEFAULT_SEPATH_PROPERTY),
    m_channelAgentSourceProperty(DEFAULT_CHANNELAGENTSOURCE_PROPERTY),
    m_channelAgentDestinationProperty(DEFAULT_CHANNELAGENTDESTINATION_PROPERTY),
    m_myproxyFtsModeProperty(DEFAULT_MYPROXYFTSMODE_PROPERTY),
    m_myproxyFtsModeRetriever(DEFAULT_MYPROXYFTSMODE_RETRIEVER),
    m_myproxyFtsModeRenewer(DEFAULT_MYPROXYFTSMODE_RENEWER){
}
                                                                                                                          
/*
 * ~SDConfig
 *
 * Destructor
 */
SDConfig::~SDConfig(){
}

/*
 * enableCache
 *
 * enable SD Cache
 */
void SDConfig::enableCache(bool b){
    if(b){
        if(0 == m_cache.get()){
            m_cache.reset(new SDCache());
        }
    } else {
        if(0 != m_cache.get()){
            m_cache.reset();
        }
    }
}
