/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: ServiceDiscoveryConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ServiceDiscoveryConfig.h,v 1.3 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SERVICEDISCOVERYCONFIG_H_
#define GLITE_DATA_AGENTS_SD_SERVICEDISCOVERYCONFIG_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {
    
/**
 * Configure ServiceDiscovery Client library
 */
class ServiceDiscoveryConfig : public glite::config::ComponentConfiguration{
protected:
    
    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finalize the Component
     */
    virtual int fini(){return 0;}

private:
    
    /*
     * Declare factory methods as friends
     */
    friend glite::config::ComponentConfiguration * ::create_glite_component();
    friend void ::destroy_glite_component(glite::config::ComponentConfiguration*);

    /**
     * Constructor
     */
    ServiceDiscoveryConfig();
        
    /**
     * Destructor
     */
    virtual ~ServiceDiscoveryConfig();

private:
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
};

} // end namespace sd
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_SD_SERVICEDISCOVERYCONFIG_H_
