/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Guid Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: GuidTestCase.cpp,v 1.3 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/Guid.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class GuidTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( GuidTestCase);
    CPPUNIT_TEST( testCreate );
    CPPUNIT_TEST( testCreateFromString );
    CPPUNIT_TEST( testCreateNull );
    CPPUNIT_TEST( testCreateError );
    CPPUNIT_TEST( testCopy_1 );
    CPPUNIT_TEST( testCopy_2 );
    CPPUNIT_TEST( testCompare_1 );
    CPPUNIT_TEST( testCompare_2 );
    CPPUNIT_TEST( testToString_1 );
    CPPUNIT_TEST( testToString_2 );
    CPPUNIT_TEST( testFromString );
    CPPUNIT_TEST( testFromStringError );
    CPPUNIT_TEST( testHash );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Create
     */
    void testCreate();
    /**
     * Test Create From String
     */
    void testCreateFromString();
    /**
     * Test Create From Empty String
     */
    void testCreateNull();
    /**
     * Test Copy
     */
    void testCopy_1();
    /**
     * Test Copy
     */
    void testCopy_2();
    /**
     * Test Compare
     */
    void testCompare_1();
    /**
     * Test Compare
     */
    void testCompare_2();
    /**
     * Test Create Error
     */
    void testCreateError();
    /**
     * Test Str
     */
    void testToString_1();
    /**
     * Test Str
     */
    void testToString_2();
    /**
     * Test From String
     */
    void testFromString();
    /**
     * Test From String
     */
    void testFromStringError();
    /**
     * Test Has and CrossSumHash functions
     */
    void testHash();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GuidTestCase, "GuidTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void GuidTestCase::setUp(){
}

/*
 * testCreate Method
 */
void GuidTestCase::testCreate(){
    
    try{
        Guid guid;
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == false);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreateFromString Method
 */
void GuidTestCase::testCreateFromString(){
    
    try{
        Guid guid("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == false);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreateNull Method
 */
void GuidTestCase::testCreateNull(){
    
    try{
        Guid guid("");
        CPPUNIT_ASSERT_MESSAGE("Not Null", guid.isNull() == true);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCopy Method
 */
void GuidTestCase::testCopy_1(){
    
    try{
        Guid guid("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid.isNull() == false);
        Guid guid2(guid);
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid2.isNull() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid2.str() == "bfecda33-6fab-4903-bb14-755344193abb");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCopy Method
 */
void GuidTestCase::testCopy_2(){
    
    try{
        Guid guid("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid.isNull() == false);
        Guid guid2;
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid2.isNull() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid2.str() != "bfecda33-6fab-4903-bb14-755344193abb");
        guid2 = guid;
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid2.isNull() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid2.str() == "bfecda33-6fab-4903-bb14-755344193abb");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCompare Method
 */
void GuidTestCase::testCompare_1(){
    
    try{
        Guid guid("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid.isNull() == false);
        Guid guid2(guid);
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid2.isNull() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid2 == guid);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCompare Method
 */
void GuidTestCase::testCompare_2(){
    
    try{
        Guid guid("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid.isNull() == false);
        Guid guid2(guid);
        CPPUNIT_ASSERT_MESSAGE("Null Guid", guid2.isNull() == false);
        if((guid2 > guid) || (guid2 < guid)){
            CPPUNIT_FAIL("Invalid Guid");
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreateError Method
 */
void GuidTestCase::testCreateError(){
    
    try{
        Guid guid("invalid");
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == true);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testToString Method
 */
void GuidTestCase::testToString_1(){
    
    try{
        Guid guid("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == false);
        std::string s = guid.str();
        CPPUNIT_ASSERT_MESSAGE("Empty Guid String", s.empty() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid String", s == "bfecda33-6fab-4903-bb14-755344193abb");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testToString Method
 */
void GuidTestCase::testToString_2(){
    
    try{
        Guid guid;
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == false);
        std::string s = guid.str();
        CPPUNIT_ASSERT_MESSAGE("Empty Guid String", s.empty() == false);
        
        Guid guid2(s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid2.isNull() == false);
        std::string s2 = guid2.str();
        CPPUNIT_ASSERT_MESSAGE("Empty Guid String", s2.empty() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid String", s2 == s);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testFromString Method
 */
void GuidTestCase::testFromString(){
    
    try{
        Guid guid("");
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == true);
        guid.fromString("bfecda33-6fab-4903-bb14-755344193abb");
        CPPUNIT_ASSERT_MESSAGE("Empty Guid", guid.isNull() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid String", guid.str() == "bfecda33-6fab-4903-bb14-755344193abb");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testFromString Method
 */
void GuidTestCase::testFromStringError(){
    
    try{
        Guid guid("");
        CPPUNIT_ASSERT_MESSAGE("Invalid Guid", guid.isNull() == true);
        guid.fromString("invalid");
        CPPUNIT_FAIL("This Guid is invalid");
    } catch(const LogicError& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testHash Method
 */
void GuidTestCase::testHash(){
    
    try{
        {
        Guid guid;
        unsigned short h = guid.hash();
        unsigned int   c = guid.crossSumHash();
        CPPUNIT_ASSERT_MESSAGE("Invalid Hash (1)", h != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid CrossSum Hash (1)", c != 0);
        }
        {
        Guid guid("");
        unsigned short h = guid.hash();
        unsigned int   c = guid.crossSumHash();
        CPPUNIT_ASSERT_MESSAGE("Invalid Hash (2)", h == 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid CrossSum Hash (2)", c == 0);
        }
        {
        Guid guid("a5cf6858-d460-4bd4-bffd-26de8791288b");
        unsigned short h = guid.hash();
        unsigned int   c = guid.crossSumHash();
        CPPUNIT_ASSERT_MESSAGE("Invalid Hash (3)", h == 2013);
        CPPUNIT_ASSERT_MESSAGE("Invalid CrossSum Hash (3)", c == 6);
        }

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
