/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: TestCommon main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestCommon.cpp,v 1.4 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */ 
#include <stdlib.h>
#include <string>
#include <iostream>
#include <unistd.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/CompilerOutputter.h>

using std::cout;
using std::cerr;
using std::cin;
using std::endl;

namespace {
const char * const PROGNAME    = "glite_data_agents_common_test_common";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";
}

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << " [-s] " << endl;
    cout << " Where:" << endl;
    cout << " -s : Enable the Compiler Outputter" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * suite
 *
 * Instantiate Test Suite
 */
CppUnit::Test * suite() {

    CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
    CppUnit::TestFactoryRegistry& urlHelpersTestCase  = CppUnit::TestFactoryRegistry::getRegistry("UrlUtilityTestCase");
    CppUnit::TestFactoryRegistry& surlHelpersTestCase = CppUnit::TestFactoryRegistry::getRegistry("SurlUtilityTestCase");
    CppUnit::TestFactoryRegistry& guidTestCase = CppUnit::TestFactoryRegistry::getRegistry("GuidTestCase");
    
    registry.registerFactory(&urlHelpersTestCase);
    registry.registerFactory(&surlHelpersTestCase);
    registry.registerFactory(&guidTestCase);

    return registry.makeTest();
}


/*
 * main
 *
 * UnitTest Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    const char * location = 0;
    bool selfTest = false;
    int flag;
    while ((flag = getopt (argc, argv, "l:sh")) != -1) {
        switch (flag) {
            case 'l':
                location = optarg;
            break;
            case 's':
                selfTest = true;
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while
    
    if(0 != location){
        // Set "GLite Location User"
        setenv("GLITE_LOCATION",location,1);
    }
    
    CppUnit::TextUi::TestRunner runner;
    runner.addTest(suite());   // Add the suite to the test runner

    if(true == selfTest){ 
        // Change the default outputter to a compiler error format outputter
        // The test runner owns the new outputter.
        runner.setOutputter( 
           CppUnit::CompilerOutputter::defaultOutputter(&runner.result(),cerr));
    }
          
    // Run the test
    bool wasSucessful = runner.run();

    return wasSucessful ? 0 : 1;
}
