/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Common: CredUtility Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CredUtilityTestCase.cpp,v 1.3 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/cred/CredServiceFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <unistd.h>
 
using namespace glite::data::agents;
using namespace glite::data::agents::cred;

using boost::scoped_ptr;

extern const char * g_user_dn;
extern const char * g_passphrase;
extern const char * g_cred_server;
extern const char * g_proxy_file;

/**
 * CPPUnit TestCase
 */
class CredUtilityTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CredUtilityTestCase );
    CPPUNIT_TEST( testGetProxyDn );
    CPPUNIT_TEST( testGetProxyCert );
    CPPUNIT_TEST( testGetProxyCert_2 );
    CPPUNIT_TEST( testGetProxyLifetime );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the GetProxyDn Method
     */
    void testGetProxyDn();
    /**
     * Test the GetProxyCert Method
     */
    void testGetProxyCert();
    /**
     * Test the GetProxyCert Method
     */
    void testGetProxyCert_2();
    /**
     * Test the GetProxyLifetime Method
     */
    void testGetProxyLifetime();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CredUtilityTestCase, "CredUtilityTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void CredUtilityTestCase::setUp(){
}

/*
 * testGetProxyDn Method
 */
void CredUtilityTestCase::testGetProxyDn(){
    
    try{
        std::string my_dn = get_proxy_dn();
        CPPUNIT_ASSERT_MESSAGE("Empty DN", false == my_dn.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetProxyDn Method
 */
void CredUtilityTestCase::testGetProxyCert(){
    
    try{
        // Get Proxy
        std::string assoc_service;
        std::string assoc_service_type;
        std::string cred_service_endpoint;
        std::string proxy_cert = get_proxy_cert(g_user_dn,g_passphrase,"VO_1",cred_service_endpoint,assoc_service,assoc_service_type,false);
        CPPUNIT_ASSERT_MESSAGE("Empty cert", false == proxy_cert.empty());

        // Delete Proxy Cert
        unlink(proxy_cert.c_str());
        
        // Change Params
        assoc_service = "filetransfer.cern.ch";
        assoc_service_type = "org.glite.FileTransfer";

        // Get Proxy
        proxy_cert = get_proxy_cert(g_user_dn,g_passphrase,"VO_1",cred_service_endpoint,assoc_service,assoc_service_type,false);
        CPPUNIT_ASSERT_MESSAGE("Empty cert", false == proxy_cert.empty());
        
        // Delete Proxy Cert
        unlink(proxy_cert.c_str());
        
        // Change Jobs Params
        cred_service_endpoint = g_cred_server;
        
        // Get Proxy
        proxy_cert = get_proxy_cert(g_user_dn,g_passphrase,"VO_1",cred_service_endpoint,assoc_service,assoc_service_type,false);
        CPPUNIT_ASSERT_MESSAGE("Empty cert", false == proxy_cert.empty());
        
        // Delete Proxy Cert
        unlink(proxy_cert.c_str());
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetProxyDn Method
 */
void CredUtilityTestCase::testGetProxyCert_2(){
    
    try{
        
        std::string assoc_service = "filetransfer.cern.ch";
        std::string assoc_service_type = "org.glite.FileTransfer";
        std::string cred_service_endpoint;
        // Get Proxy
        std::string proxy_cert = get_proxy_cert(g_user_dn,g_passphrase,"VO_1",cred_service_endpoint,assoc_service,assoc_service_type,false);
        CPPUNIT_ASSERT_MESSAGE("Empty cert", false == proxy_cert.empty());

        // Delete Proxy Cert
        unlink(proxy_cert.c_str());
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetProxyLeffetime Method
 */
void CredUtilityTestCase::testGetProxyLifetime(){
    
    try{
        
        // Get Proxy Lifetime
        time_t lifetime = get_proxy_lifetime(g_proxy_file);
        CPPUNIT_ASSERT_MESSAGE("Expired Proxy", lifetime > 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
