/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: TestCredService main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestCredService.cpp,v 1.4 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
 
#include "glite/data/config/service/glite-config.h"
#include "glite/data/config/service/glite-config-register.hxx"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <unistd.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/CompilerOutputter.h>



using std::cout;
using std::cerr;
using std::cin;
using std::endl;

namespace {
const char * const PROGNAME    = "glite_data_agent_common_test_cred";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";

const char * const DEFAULT_CONFIG_NAME = "glite-agents-common-test-cred";
}

/*
 * Define Global variables
 */
const char * g_user_dn     = 0;
const char * g_passphrase  = 0;
const char * g_cred_server = 0;
const char * g_proxy_file = 0;

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << " -u< user_dn> -p <passphrase> -m <endpoint> -f <proxy_file> [-l <glite_location_user>][-c <config_name>][-s]" << endl;
    cout << " Where:" << endl;
    cout << " -u : Specify the DN of the user" << endl;
    cout << " -p : Specify the passphrase for the user certificate" << endl;
    cout << " -m : Specify the cred server endpoint" << endl;
    cout << " -f : Specify a valid proxy file" << endl;
    cout << " -l : Specify the glite location. Configuration files should be located under the etc folder on that path" << endl;
    cout << " -c : Configuration Name" << endl;
    cout << " -s : Enable the Compiler Outputter" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * suite
 *
 * Instantiate Test Suite
 */
CppUnit::Test * suite() {

    CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
    CppUnit::TestFactoryRegistry& credServiceTestCase = CppUnit::TestFactoryRegistry::getRegistry("CredServiceTestCase");
    CppUnit::TestFactoryRegistry& credUtilityTestCase = CppUnit::TestFactoryRegistry::getRegistry("CredUtilityTestCase");
    
    registry.registerFactory(&credServiceTestCase);
    registry.registerFactory(&credUtilityTestCase);

    return registry.makeTest();
}


/*
 * main
 *
 * UnitTest Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    const char * location = 0;
    const char * config_name = DEFAULT_CONFIG_NAME;
    bool selfTest = false;
    int flag;
    while ((flag = getopt (argc, argv, "u:p:l:m:c:f:sh")) != -1) {
        switch (flag) {
            case 'u':
                g_user_dn = optarg;
            break;
            case 'p':
                g_passphrase = optarg;
            break;
            case 'm':
                g_cred_server = optarg;
            break;
            case 'f':
                g_proxy_file = optarg;
            break;
            case 'l':
                location = optarg;
            break;
            case 'c':
                config_name = optarg;
            break;
            case 's':
                selfTest = true;
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while

    if((0 == g_user_dn)     || (0 == strlen(g_user_dn))    ||
       (0 == g_passphrase)  || (0 == strlen(g_passphrase)) ||
       (0 == g_cred_server) || (0 == strlen(g_cred_server))||
       (0 == g_proxy_file)  || (0 == strlen(g_proxy_file))){
        print_usage_and_die();
    }
    
    if(0 != location){
        // Set "GLite Location"
        setenv("GLITE_LOCATION",location,1);
    }
    
    // Initialize the Application
    int initres = glite_config_initialize(config_name);
    if(initres < 0){
        cerr << "Initialization failed " << endl;
        return -1;
    }
    
    CppUnit::TextUi::TestRunner runner;
    runner.addTest(suite());   // Add the suite to the test runner

    if(true == selfTest){ 
        // Change the default outputter to a compiler error format outputter
        // The test runner owns the new outputter.
        runner.setOutputter( 
           CppUnit::CompilerOutputter::defaultOutputter(&runner.result(),cerr));
    }
          
    // Run the test
    bool wasSucessful = runner.run();

    // Finalize the Application
    glite_config_finalize();
        
    return wasSucessful ? 0 : 1;
}
