/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: ActiveObject Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActiveObjectTestCase.cpp,v 1.4 2007/01/24 12:08:24 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/ActiveObject.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * TestActiveObject
 */
class TestActiveObject : public ActiveObject{
public:
    /**
     * Constructor
     */
    TestActiveObject():ActiveObject("Test"),m_count(0){}
    /**
     * Destructor
     */
    virtual ~TestActiveObject(){}
    /**
     * Get Count Property
     */
    unsigned long count() const {return m_count;}
protected:
    /**
     * Object Main Function
     */
    virtual void svc() throw(){
        while(this->doRun()){
            CriticalLock lock(*this);
            m_count++;
            sleep(1);
        }
    }
private:    
    /**
     * Count Property
     */
    unsigned long m_count;
};

/**
 * CPPUnit TestCase
 */
class ActiveObjectTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ActiveObjectTestCase);
    CPPUNIT_TEST( test_1 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the GetSiteForSurl Method
     */
    void test_1();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ActiveObjectTestCase, "ActiveObjectTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void ActiveObjectTestCase::setUp(){
}

/*
 * Test Method
 */
void ActiveObjectTestCase::test_1(){
    
    try{
        // Create ActiveObject
        TestActiveObject obj;
        CPPUNIT_ASSERT_MESSAGE("Invalid Id", obj.id() == (ActiveObject::id_type)-1);
        // Start ActiveObject
        bool s = obj.start();
        CPPUNIT_ASSERT_MESSAGE("Not started", s == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Id", obj.id() != (ActiveObject::id_type)-1);
        // Sleep for a while
        sleep(2);
        // Stop ActiveObject
        obj.stop();
        CPPUNIT_ASSERT_MESSAGE("Invalid Id", obj.id() == (ActiveObject::id_type)-1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Count", obj.count() > 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
