/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Extra Utility Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ExtraUtilityTestCase.cpp,v 1.5 2007/01/24 12:08:24 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/utility.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class ExtraUtilityTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ExtraUtilityTestCase);
    CPPUNIT_TEST( testGetSiteForSurl_1 );
    CPPUNIT_TEST( testGetSiteForSurl_2 );
    CPPUNIT_TEST( testGetSiteForSurl_3 );
    CPPUNIT_TEST( testGetSiteForSurl_4 );
    CPPUNIT_TEST( testGetSiteForSurl_5 );
    CPPUNIT_TEST( testStrToSize );
    CPPUNIT_TEST( testStrToSize_Error );
    CPPUNIT_TEST( testHashString );
    CPPUNIT_TEST( testGetDirName );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the GetSiteForSurl Method
     */
    void testGetSiteForSurl_1();
    void testGetSiteForSurl_2();
    void testGetSiteForSurl_3();
    void testGetSiteForSurl_4();
    void testGetSiteForSurl_5();
    /**
     * Test the StrToSize Method
     */
    void testStrToSize();
    void testStrToSize_Error();
    /**
     * Test the HashString Method
     */
    void testHashString();
    /**
     * Test the GetDirName Method
     */
    void testGetDirName();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ExtraUtilityTestCase, "ExtraUtilityTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void ExtraUtilityTestCase::setUp(){
}

/*
 * Test testGetSiteForSurl Method
 */
void ExtraUtilityTestCase::testGetSiteForSurl_1(){
    
    try{
        std::string vo;
        std::string surl = "srm://castorsrm.cern.ch/castor/cern.ch/test_file.txt";
        std::string site = get_site_for_surl(surl, vo, false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site (0)", site == "CERN.CH");
        
        surl = "srm://castorsrm.cern.ch:8443/castor/cern.ch/test_file.txt";
        site = get_site_for_surl(surl, vo, false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site (1)", site == "CERN.CH");

        surl = "srm://castorsrm.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/test_file.txt";
        site = get_site_for_surl(surl, vo, false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site (2)", site == "CERN.CH");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetSiteForSurl Method
 */
void ExtraUtilityTestCase::testGetSiteForSurl_2(){
    
    try{
        std::string vo = "VO_1";
        std::string surl = "srm://gridftp05.cern.ch/castor/cern.ch/test_file.txt";
        std::string site = get_site_for_surl(surl, vo, false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site (0)", site == "CERN.CH");
        
        surl = "srm://gridftp05.cern.ch:8443/castor/cern.ch/test_file.txt";
        site = get_site_for_surl(surl, vo, false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site (1)", site == "CERN.CH");

        surl = "srm://gridftp05.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/test_file.txt";
        site = get_site_for_surl(surl, vo, false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site (2)", site == "CERN.CH");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetSiteForSurl Method
 */
void ExtraUtilityTestCase::testGetSiteForSurl_3(){
    
    try{
        std::string vo = "VO_1";
        std::string surl = "srm://gridftp06.cern.ch/castor/cern.ch/test_file.txt";
        std::string site = get_site_for_surl(surl, vo, false);
        CPPUNIT_FAIL("This SE should not be visible by VO 1");
    } catch(const RuntimeError& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetSiteForSurl Method
 */
void ExtraUtilityTestCase::testGetSiteForSurl_4(){
    
    try{
        std::string vo;
        std::string surl = "srm://unknown.cern.ch/castor/cern.ch/test_file.txt";
        std::string site = get_site_for_surl(surl, vo, false);
        CPPUNIT_FAIL("This SE should not exist");
    } catch(const RuntimeError& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
    
    try{
        std::string vo;
        std::string surl = "srm://unknown.cern.ch/castor/cern.ch/test_file.txt";
        std::string site = get_site_for_surl(surl, vo, true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site ", site == "UNKNOWN");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetSiteForSurl Method
 */
void ExtraUtilityTestCase::testGetSiteForSurl_5(){
    
    try{
        std::string vo;
        std::string surl = "/castor/cern.ch/test_file.txt";
        std::string site = get_site_for_surl(surl, vo, true);
        CPPUNIT_FAIL("This SURL is invalid");
    } catch(const LogicError& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test str_to_size Method
 */
void ExtraUtilityTestCase::testStrToSize(){
    
    try{
        unsigned long size = str_to_size("1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (1)", size == 1);
        size = str_to_size("123456");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (2)", size == 123456);
        size = str_to_size("    1234567    ");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (3)", size == 1234567);
        size = str_to_size("1 KB");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (4)", size == 1024);
        size = str_to_size("1GB");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (5)", size == (1024 * 1024 * 1024));
        size = str_to_size("1.5 MB");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (6)", size == (1024 * 1536));
        size = str_to_size(".5KB");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (7)", size == 512);
        size = str_to_size("  .5   KB    ");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (8)", size == 512);
        size = str_to_size("");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (9)", size == 0);
        size = str_to_size("    ");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (10)", size == 0);
        size = str_to_size("	12345678	");
        CPPUNIT_ASSERT_MESSAGE("Invalid Size (11)", size == 12345678);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test str_to_size Method
 */
void ExtraUtilityTestCase::testStrToSize_Error(){
    
    try{
        unsigned long size;
        try{
            size = str_to_size("a112");
            CPPUNIT_FAIL("Invalid Format (1)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("123a");
            CPPUNIT_FAIL("Invalid Format (2)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("123 1");
            CPPUNIT_FAIL("Invalid Format (3)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("123.1");
            CPPUNIT_FAIL("Invalid Format (4)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("1.12.1");
            CPPUNIT_FAIL("Invalid Format (5)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("    1a122");
            CPPUNIT_FAIL("Invalid Format (6)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("   a12");
            CPPUNIT_FAIL("Invalid Format (7)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("1. 12");
            CPPUNIT_FAIL("Invalid Format (8)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("1 M");
            CPPUNIT_FAIL("Invalid Format (9)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("1 TB");
            CPPUNIT_FAIL("Invalid Format (10)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("1.222 FOO");
            CPPUNIT_FAIL("Invalid Format (11)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("1 KB MB");
            CPPUNIT_FAIL("Invalid Format (12)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("-10");
            CPPUNIT_FAIL("Invalid Format (13)");
        } catch(const LogicError& exc){
            // OK
        } 
        try{
            size = str_to_size("11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111");
            CPPUNIT_FAIL("Invalid Format (13)");
        } catch(const LogicError& exc){
            // OK
        } 
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test HashString Method
 */
void ExtraUtilityTestCase::testHashString(){
    
    try{
        std::string str1 =  "Nel mezzo del cammin di nostra vita "\
                                "mi ritrovai per una selva oscura "\
                            "ché la diritta via era smarrita.";
        std::string str2 =      "Ahi quanto a dir qual era è cosa dura "\
                            "esta selva selvaggia e aspra e forte "\
                            	"che nel pensier rinova la paura!";
        std::string str3 =  "Tant'è amara che poco è più morte; "\
                            	"ma per trattar del ben ch'i' vi trovai, "\
                        	"dirò de l'altre cose ch'i' v'ho scorte.";
        unsigned long h1,h2,h3;
        h1 = hash_string(str1);
        CPPUNIT_ASSERT_MESSAGE("Invalid hash (1)", h1 != 0);
        h2 = hash_string(str2);
        CPPUNIT_ASSERT_MESSAGE("Invalid hash (2)", h2 != 0);
        CPPUNIT_ASSERT_MESSAGE("non-unique hash (2-1)", h2 != h1);
        h3 = hash_string(str3);
        CPPUNIT_ASSERT_MESSAGE("Invalid hash (3)", h3 != 0);
        CPPUNIT_ASSERT_MESSAGE("non-unique hash (3-1)", h3 != h1);
        CPPUNIT_ASSERT_MESSAGE("non-unique hash (3-2)", h3 != h2);
    } catch(const LogicError& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test GetDirName Method
 */
void ExtraUtilityTestCase::testGetDirName(){
                                                                                                                        
    try{
        std::string path = "/tmp/test/agents-common/testfile";
        std::string dir_name = get_dirname(path);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (1)", dir_name == "/tmp/test/agents-common");
        
        path = "/tmp/test/agents-common/testfile/";
        dir_name = get_dirname(path);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (2)", dir_name == "/tmp/test/agents-common");
        
        path = "/tmp";
        dir_name = get_dirname(path);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (3)", dir_name == "/");
        
        path = "/";
        dir_name = get_dirname(path);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (4)", dir_name == "/");
    } catch(const LogicError& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
