/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <glite/data/agents/extra/options.h>
#include <glite/data/agents/glite-types.h>

using namespace glite::data::agents::options;

static int flag_one = 0;
static int flag_two = 0;
static const char * opt_x = 0;
static char * opt_y = 0;
static const char * opt_one = 0;
static const char * opt_two = 0;
static const char * opt_long = 0;

void set_f1(char*){
    flag_one = 1;
}
void set_o1(char* arg){
    opt_one = arg;
}
void set_o2(char* arg){
    opt_two = arg;
}
void set_l(char* arg){
    opt_long = arg;
}

int main(int argc,char *argv[]){
    
    // create args object
    //--------------------------------------------------------------------------
    Arguments args(argc,argv);
    try{
        opt(set_f1,         args,'1',"flag-one")    .help("set the first flag.");
        opt(flag_two,   1,  args,'2',"flag-two")    .help("set the second flag.");
        opt(opt_y,          args,'y')               .arg("Y").help("set the y flag.");
        opt(opt_x,          args,'x')               .arg("X").help("set the x flag.")
                                                    .help("The x flag does nothing, like all other flags,")
                                                    .help("but is used here to test help strings spanning")
                                                    .help("multiple lines.");
        opt(set_o1,         args,   "option-one")   .arg("ONE")
                                                    .help("set option one to ONE");
        opt(set_o2,         args,   "option-two")   .arg("TWO")
                                                    .help("set option two to TWO");
        opt(set_l,          args,'l',"an-option-with-a-very-long-name").arg()
                                                    .help("test line wrapping in help message.");
        args.pos("first-positional")                .help("the first positional parameter.")
                                                    .help("Also this help message spans multiple lines.");
        args.pos("the-second-positional-parameter-with-a-very-long-name")
                                                    .help("test line wrapping in help message once again.");
    } catch (const std::exception& exc){
        std::cerr << exc.what() << std::endl;
        return 1;
    }
    
    if (1 == argc){
        std::cout << "Command line options test program : " << __DATE__ << " - " << __TIME__ << std::endl;
        args.showHelp();
        return 0;
    }
    
    // tests    
    //--------------------------------------------------------------------------
    // test duplicate options insertion
    int tmp = 0;
    try{
        std::cerr << "Checking insertion of duplicate options (short)... ";
        opt(tmp,1,args,'1');
        std::cerr << "ERROR!" << std::endl;
        return 1;
    } catch (const std::exception& exc){
        std::cerr << "OK (" << exc.what() << ")" << std::endl;
    }
    try{
        std::cerr << "Checking insertion of duplicate options (long)... ";
        opt(tmp,1,args,"flag-one");
        std::cerr << "ERROR!" << std::endl;
        return 1;
    } catch (const std::exception& exc){
        std::cerr << "OK (" << exc.what() << ")" << std::endl;
    }
        
    // parse options and show vlaues    
    //--------------------------------------------------------------------------
    if (!args.parse()){
        args.showHelp();
        return 1;
    }
    std::cout << "flag_one = " << flag_one << std::endl;
    std::cout << "flag_two = " << flag_two << std::endl;
    #define PROPT(a) std::cout << #a << " = "; a ? std::cout << a : std::cout << "NULL";std::cout << std::endl;
    PROPT(opt_x);
    PROPT(opt_y);
    PROPT(opt_one);
    PROPT(opt_two);
    PROPT(opt_long);
    for (;optind < argc;++optind){
        std::cout << "positional parameter: " << argv[optind] << std::endl;
    }
    return 0;
}

