/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: 19357 Unit Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestCase19357.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/cred/CredServiceFactory.h"
#include "glite/data/agents/cred/CredServiceException.h"

#include <glite/data/config/service/Logger.hxx>

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;
using namespace glite::data::agents::cred;


using boost::scoped_ptr;

extern const char * g_user_dn;
    
/**
 * CPPUnit TestCase
 */
class TestCase19357 : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    /**
     * tear-down test case
     */
    void tearDown ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TestCase19357 );
    CPPUNIT_TEST( test1 );
    CPPUNIT_TEST( test2 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Methods
     */
    void test1();
    void test2();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TestCase19357, "TestCase19357" );

/*
 * Declare Constants. This unit test foresee at beginning
 */
namespace {
}

/*
 * setUp
 *
 * set-up test case
 */
void TestCase19357::setUp(){
    
    try{
         // TODO
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * tearDown
 *
 * tear-down test case
 */
void TestCase19357::tearDown(){
}

/*
 *
 * Test Execute Method
 */
void TestCase19357::test1(){
    
    try{
        CredServiceFactory& factory = CredServiceFactory::instance();
        
        // Create CertProxyService Instance
        scoped_ptr<CredService> proxy(factory.create(""));
        
        // Get the credential with an empty 
        std::string f;
        proxy->get(g_user_dn,"",f);
        // This should throw an exception !
        CPPUNIT_FAIL("Empty passphrase!");
    } catch(const LogicError& exc){
        std::string reason = exc.what();
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason", false == reason.empty());
    } catch(const CredServiceException& exc){
        std::string reason = exc.what();
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason", false == reason.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 *
 * Test Execute Method
 */
void TestCase19357::test2(){
    
    try{
        std::string assoc_service;
        std::string assoc_service_type;
        std::string cred_service_endpoint;
        std::string proxy_cert = get_proxy_cert(g_user_dn,"","VO_1",cred_service_endpoint,assoc_service,assoc_service_type,false);
        // This should throw an exception !
        CPPUNIT_FAIL("Empty passphrase!");
    } catch(const LogicError& exc){
        std::string reason = exc.what();
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason", false == reason.empty());
    } catch(const CredServiceException& exc){
        std::string reason = exc.what();
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason", false == reason.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
