/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: TestSD main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: StressSD.cpp,v 1.5 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
 
#include "ServiceDiscoveryIfce.h"
#include <stdlib.h>
#include <string>
#include <iostream>
#include <unistd.h>

#include "glite/data/glite-util.h"

using std::cout;
using std::cerr;
using std::cin;
using std::endl;

namespace {
const char * const CONFIG_NAME = "glite-transfer-agent-test-stress-sd";
const char * const PROGNAME    = "glite_data_transfer_agent_test_stress_sd";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";
}

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << " [-n <iter>] " << endl;
    cout << " Where:" << endl;
    cout << " -n : Number of iterations" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * do_iteration
 *
 * iteartion function
 */
int do_iteration() {

    // Get the vo List
    SDVOList *vos = 0;
    
    // List Services
    SDException exception;
    SDServiceList * service_list = SD_listServices("SRM", 0, vos, &exception);
    if(0 == service_list){
        const char * reason = (0 != exception.reason)?exception.reason:"(null)";
        cerr << "No SRM services. Reson is " << reason << endl;
        SD_freeException(&exception);
        return -1;
    }

    // Loop through the services
    if(service_list->numServices > 0){
       
        for(int i = 0; i < service_list->numServices; i++){
            SDService * sd_service = service_list->services[i];
            if(0 == sd_service){
                continue;
            }
            
            // Get Service Details
            SDException exception;
            SDServiceDetails * sd_details = SD_getServiceDetails(sd_service->name, &exception);
            if(0 == sd_details){
               const char * name   = (0 != sd_service->name)?sd_service->name:"(null)";
               const char * reason = (0 != exception.reason)?exception.reason:"(null)";
               cerr << "No service details for the service " << name << ". Reason is " << reason << endl;
               SD_freeException(&exception);
            } 
            // Free Memory
            SD_freeServiceDetails(sd_details);
        } // end for
    } 
    
    // Free Memory
    SD_freeServiceList(service_list);
    
    return 0;
}


/*
 * main
 *
 * Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    unsigned int iter_num = 10000;
    int flag;
    while ((flag = getopt (argc, argv, "n:h")) != -1) {
        switch (flag) {
            case 'n':
                iter_num = atoi(optarg);
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while

    if(iter_num <= 0){
        print_usage_and_die();
    }

    for(unsigned int i = 0; i < iter_num; ++i){
        do_iteration();
        if(0 == (i % 1000)){
            cout << ".";
            cout.flush();
            sleep(1);
        }
    }        

    return 0;
}
